unit plugin;

interface

uses Windows, SysUtils, Forms, Messages, Dialogs;

var
 ccdok: Boolean;

function d_Initialize(whandle: HWND): Integer; stdcall;
function d_Finalize: Integer; stdcall;
function d_SetupCCD(silent: Integer): Integer; stdcall;
function d_GetOK: Integer; stdcall;
function d_Subframe(x,y,w,h,binmode: Integer): Integer; stdcall;
function d_MoveFrame(param:Integer): Integer; stdcall;
function d_GetWidth(param:Integer): Integer; stdcall;
function d_GetHeight(param:Integer): Integer; stdcall;
function d_BinningCount: Integer; stdcall;
function d_BinningName(n: Integer): PAnsiChar; stdcall;
function d_GetName: PAnsiChar; stdcall;
function d_GetInfo: PAnsiChar; stdcall;
function d_GetTemperature: Integer; stdcall;
function d_StartExposure(expo,openshutter:Integer): Integer; stdcall;
function d_IsExposing: Integer; stdcall;
function d_StartDownload: Integer; stdcall;
function d_ReadImage(pbuffer:Pointer): Integer; stdcall;
function d_EndDownload(pbuffer:Pointer): Integer; stdcall;
function d_InterruptExposure: Integer; stdcall;
function d_IsColor: Integer; stdcall;
function d_SetTemperature(milliCelsius,coolerPerc:Integer): Integer; stdcall;


implementation


uses fccd;

var
 ccdexposing: Boolean;
 endtick,ccdx,ccdy,ccdw,ccdh,ccdb: Integer;
 temperature: Integer = -12500;
 statstring: AnsiString;

const
 MV_UP = 1;
 MV_DOWN = 2;
 MV_LEFT = 3;
 MV_RIGHT = 4;

type
 TWordArray = array [0..999999999] of word;
 PWordArray = ^TWordArray;

function d_Initialize(whandle: HWND): Integer; stdcall;
begin
  ccdok := false;
  FormCCD := nil;
  result := 1;
end;

function d_Finalize: Integer; stdcall;
begin
  if FormCCD <> nil then FormCCD.Release;
  result := 1;
end;

function d_SetupCCD(silent: Integer): Integer; stdcall;
begin
  if FormCCD = nil then FormCCD := TFormCCD.Create(Application);
  if silent = 0 then FormCCD.ShowModal
  else begin
     FormCCD.Show;
     FormCCD.ButOK.Click;
  end;
  result := 1;
end;

function d_GetOK: Integer; stdcall;
begin
  if ccdok then result := 1 else result := 0;
end;

function d_Subframe(x,y,w,h,binmode: Integer): Integer; stdcall;
var binning,chipx,chipy,scale: Integer;
begin
  result := 0;
  chipx := 500;    // Chip has 500x350 pixels
  chipy := 350;
  case binmode of
   0: binning := 1;
   1: binning := 2;
   2: binning := 4;
   else exit;
  end;
  scale := 100000;    // Fixed percentage scale factor
  ccdb := binning;
  ccdw := (chipx div binning) * w div scale;
  ccdh := (chipy div binning) * h div scale;
  ccdx := (chipx div binning) * x div scale;
  ccdy := (chipy div binning) * y div scale;
  result := 1;
end;

function d_MoveFrame(param:Integer): Integer; stdcall;
begin
  case param of
   MV_UP: inc(ccdy);
   MV_DOWN: dec(ccdy);
   MV_LEFT: dec(ccdx);
   MV_RIGHT: inc(ccdx);
  end;
  result := 1;
end;

function d_GetWidth(param:Integer): Integer; stdcall;
begin
  result := ccdw;
end;

function d_GetHeight(param:Integer): Integer; stdcall;
begin
  result := ccdh;
end;

function d_BinningCount: Integer; stdcall;
begin
  result := 3;  // Three binning modes available: 1x1, 2x2, 4x4
end;

function d_BinningName(n: Integer): PAnsiChar; stdcall;
begin
  case n of
   0: result := '1 X 1';
   1: result := '2 X 2';
   2: result := '4 X 4';
   else result := nil;
  end;
end;

function d_GetName: PAnsiChar; stdcall;
begin
  result := 'Demo CCD Camera - Control panel';
end;

function d_GetInfo: PAnsiChar; stdcall;
begin
  statstring := Format('T = %4.1f  50%%',[temperature/1000.0]);
  result := PAnsiChar(statstring);
end;

function d_GetTemperature: Integer; stdcall;
begin
  result := temperature;   // (Celsius * 1000)
end;

function d_SetTemperature(milliCelsius,coolerPerc:Integer): Integer; stdcall;
begin
  temperature := milliCelsius;   // (Celsius * 1000)
  result := 1;
end;

function d_StartExposure(expo,openshutter:Integer): Integer; stdcall;
begin
  ccdexposing := true;
  endtick := Integer(GetTickCount) + expo;
  result := 1;
end;

function d_IsExposing: Integer; stdcall;
begin
  result := 0;
  if not ccdexposing then exit;
  if Integer(GetTickCount) > endtick then begin
     ccdexposing := false;
     exit;
  end;
  result := 1;
end;

function d_StartDownload: Integer; stdcall;
begin
  Sleep(ccdw * ccdh div 100 + 100);  // Simulates a download delay
  result := 1;
end;

function d_ReadImage(pbuffer:Pointer): Integer; stdcall;
var
 x,y,c: Integer;
 pdata: pWordArray;
begin
  c := 0;
  pdata := pbuffer;
  for y := ccdh-1 downto 0 do     // Simulates a real image
  for x := 0 to ccdw-1 do begin
      pdata[c] := GetCCDPixel((x+ccdx)*ccdb,(y+ccdy)*ccdb);
      inc(c);
  end;
  result := 1;
end;

function d_EndDownload(pbuffer:Pointer): Integer; stdcall;
begin
  result := 1;
end;

function d_InterruptExposure: Integer; stdcall;
begin
  ccdexposing := false;
  result := 1;
end;

function d_IsColor: Integer; stdcall;
begin
  result := 0;
end;

(*
function d_Telescope(cmd,t: Integer); stdcall;     // Optional function

// "cmd" is a number [0..11] which can be defined by the following C enum:
{te_present, te_name, te_initialize, te_finalize, te_n, te_s, te_e, te_w, te_ne, te_nw, te_se, te_sw}

// Here is a possible implementation, "relays" is a utility function:

if (t < 0) t = 1000; // t negative disabled by you.
// t negative could be supported, it means "activate the relay
// and keep it active forever and exit immediately", it's used by the the
// virtual keypad of Astroart. if t is zero then the pulse is a "stop" command

ok = 1;
switch (c) {
 case te_present: ok = 1; break;
 case te_name: ok = (int)(*ptelescopestr) ....
      te_initialize: ....
      te_finalize: ....
      te_n: relays(t,0,0,0); ....
      te_s: relays(0,t,0,0); ....
      te_e: relays(0,0,t,0);
      te_w: relays(0,0,0,t);
      te_ne: relays(t,0,t,0);
      te_se: relays(0,t,t,0);
      te_nw: relays(t,0,0,t);
      te_sw: relays(0,t,0,t);
 default ok = 0;
}
return ok
};

function d_FilterWheel(cmd,param: Integer); stdcall;  // Optional function

 // "cmd" is a number [0..5] which can be defined by the following C enum:
 // {fw_present,fw_name,fw_initialize,fw_finalize,fw_setpos,fw_getpos}

 int ok;
 char *FWName = "DemoCamera Filter Wheel";
 static int lastpos; 

 ok = 1;
 switch (cmd) {
  case fw_present: ok = 8; break;      // change to ok = 0 to disable
  case fw_name: ok = (int)FWName; break;
  case fw_initialize: lastpos = 1; break;
  case fw_finalize: break; 
  case fw_setpos: lastpos = param; break;
  case fw_getpos: ok = lastpos; break;
  default: ok = 0;
 }
 return ok;

*)


end.

