//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

//---------------------------------------------------------------------------
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
    return 1;
}
//---------------------------------------------------------------------------

enum AstroartCommand {ac_getbuffer, ac_getsize, ac_redraw, ac_close, ac_savefits,
   ac_copy, ac_writeheader, ac_readheader, ac_prepareundo, ac_thresholds, ac_transferfunc,
   ac_getselpoint, ac_getselpoint2, ac_gethwnd, ac_create, ac_open, ac_getimage,
   ac_getversion, ac_rename, ac_getname, ac_getra, ac_getdec,ac_getx, ac_gety, ac_modified,
   ac_blink, ac_flip, ac_resize, ac_findcoordinates, ac_reserved_1, ac_selectpoint, ac_selectrect, 
   ac_reserved_2, ac_reserved_3, ac_reserved_4, ac_centeratlas};

/* Commands: see docs

NAME             PARAMS TYPE                   RESULT

ac_getbuffer    (pimage, &buffer, colorplane)   0,2
ac_getsize      (pimage, &x, &y)                0,1
ac_redraw       (pimage, calcstats, autovis)    0,1
ac_close        (pimage, askuser, NULL)         0,1
ac_savefits     (pimage, pfilename, NULL)       0,1
ac_copy         (pimage, NULL, NULL)            0,1
ac_writeheader  (pimage, pheader, NULL)         0,1
ac_readheader   (pimage, NULL, NULL)            pheader
ac_prepareundo  (pimage, NULL, NULL)            0,1
ac_thresholds   (pimage, min, max)              0,1
ac_transferfunc (pimage, trf, NULL)             0,1
ac_getselpoint  (pimage, &x, &y)                0,1
ag_getselpoint2 (pimage, &x, &y)                0,1
ac_gethwnd      (pimage, &HWND, NULL)           0,1
ac_create       (x,y,colored)                   pimage
ac_open         (pfilename, NULL, NULL)         pimage
ac_getimage     (selection, NULL, NULL)         pimage

ac_getversion   (NULL, NULL, NULL)              300+
ac_rename       (pimage, name, path)            0,1
ac_getname      (pimage, keeppath, NULL)        pname
ac_getra        (pimage, x*100, y*100)          ra*100000
ac_getdec       (pimage, x*100, y*100)          dec*100000
ac_getx         (pimage, ra*1000000, dec*1000000) x*100
ac_gety         (pimage, ra*1000000, dec*1000000) y*100
ac_modified     (pimage, modified, nil)         0,1

ac_blink        (NULL, NULL, NULL)              0,1
ac_flip         (pimage, axis, NULL)            0,1
ac_resize       (pimage, dimx, dimy)            0,1
ac_findcoordinates (pimage, pparameters, NULL)  0,1
ac_selectpoint  (pimage, x, y)                  0,1
ac_selectrect   (pimage, prect, NULL)           0,1
ac_centeratlas  (ra*1E6, dec*1E6, showAtlas)    1

*/

typedef int WINAPI(*Tfcallback) (AstroartCommand, void*, void*, void*);

Tfcallback fcallback;

int PACTIVEIMAGE = 4096;
int PSELECTIMAGE = 4097;
int PEQU_STD = 1000;
int PEQU_GAU = 1001;
int PCGREEN = 0;
int PCRED = 2;
int PCBLUE = 3;

int dimx,dimy;
float *newbuffer, *oldbuffer;

// Utility functions. Not optimized, for learning purposes.

float getPixel(int x, int y)
{
 if (x<0) x = 0;
 if (y<0) y = 0;
 if (x>=dimx) x = dimx-1;
 if (y>=dimy) y = dimy-1;
 return oldbuffer[x+y*dimx];
}

void setPixel(int x, int y, float v)
{
 if (x<0 || y<0 || x >= dimx || y >= dimy) return;
 newbuffer[x+y*dimx] = v;
}

// Functions exported by the DLL

extern "C" __declspec(dllexport)
int WINAPI pi_initialize(Tfcallback funcaddress, char *menuname,
                         char *description, HWND whandle)
{
 strcpy(menuname,"C++ Hi-pass Demo");
 strcpy(description,"C++ Demo plug-in for Astroart 4.0/5.0; a high pass filter");
 fcallback = funcaddress;
 return 1;
}

extern "C" __declspec(dllexport)
void WINAPI pi_finalize(HWND whandle)
{
 // optional clean-up
}

extern "C" __declspec(dllexport)
void WINAPI pi_activate(void *pimage, HWND whandle)
{
 int x,y,b;
 float v;

 if (pimage == NULL) return;                           // no images opened
 fcallback(ac_getsize,pimage,&dimx,&dimy);             // get image size
 fcallback(ac_prepareundo,pimage,NULL,NULL);
 oldbuffer = (float*) SysGetMem(dimx*dimy*4);          // temp buffer


 b = fcallback(ac_getbuffer,pimage,&newbuffer,(void*)PCGREEN);  //***
 if (b != 2) return;                         // buffer is not 32 bit, old version of Astroart
 memcpy(oldbuffer,newbuffer,dimx*dimy*4);    // the other buffer
 for (y=0; y<dimy; y++)                    // convolution mask:     0 -1  0
  for (x=0; x<dimx; x++)                                      //   -1  6 -1
  {                                                           //    0 -1  0
  v = (getPixel(x,y)*6 - getPixel(x+1,y) - getPixel(x,y+1)
       - getPixel(x-1,y) - getPixel(x,y-1)) / 2;              // 2 = 6-1-1-1-1
  setPixel(x,y,v);
  }
 //*** on color images you should repeat these steps for the other color planes.

 SysFreeMem(oldbuffer);
 fcallback(ac_redraw,pimage,(void*)1,(void*)0);   // update the filtered image
}  


// Take a look at the following variation:
// It creates an another image instead overwriting the original.


/*
extern "C" __declspec(dllexport)
void WINAPI pi_activate(void *pimage, HWND whandle)
{
 int x,y,b,im;
 float v;

 if (pimage == NULL) return;
 fcallback(ac_getsize,pimage,&dimx,&dimy);               // get image size
 b = fcallback(ac_getbuffer,pimage,&oldbuffer,NULL);     // get image buffer

 if (b != 2) return;   // buffer is not 32 bit data

 im = fcallback(ac_create,(void*)dimx,(void*)dimy,NULL); // create a new image
 fcallback(ac_getbuffer,(void*)im,&newbuffer,NULL);      // get new buffer

 for (y=0; y<dimy; y++)                    // convolution mask:     0 -1  0
  for (x=0; x<dimx; x++)                                      //   -1  6 -1
  {                                                           //    0 -1  0
  v = (getPixel(x,y)*6 - getPixel(x+1,y) - getPixel(x,y+1)
       - getPixel(x-1,y) - getPixel(x,y-1)) / 2;              // 2 = 6-1-1-1-1
  setPixel(x,y,v);
  }

 fcallback(ac_redraw,(void*)im,(void*)1,(void*)1);   // update the filtered image
}
*/


