﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;

namespace AARemoteControl
{

public partial class Form1 : Form
{
    [DllImport("AARemote.dll")]
    public static extern int AASendCommand(int command, string parameters);
    [DllImport("AARemote.dll")]
    public static extern string AAReadScriptVariable(string varName);
    [DllImport("AARemote.dll")]
    public static extern int AAGetPixel(int x, int y);
    [DllImport("AARemote.dll")]
    public static extern int AASetPixel(int x, int y, int v);


    Bitmap bmp;

    public Form1()
    {
        InitializeComponent();
        bmp = new Bitmap(16, 16, PixelFormat.Format32bppRgb);
    }

    private void butOpen_Click(object sender, EventArgs e)
    {
        AASendCommand(CMD.EXECUTE, "");
        AASendCommand(CMD.SELECT_INTERFACE_62, "");  // remove this for 6.0
    }

    private void butClose_Click(object sender, EventArgs e)
    {
        AASendCommand(CMD.TERMINATE, "");
    }

    private bool canRunScript()
    {
        int status = AASendCommand(CMD.SCRIPT_GETSTATUS, "");
        if (status != STS.IDLE)
        {
            MessageBox.Show("Error: the previous script is still running.");
            return false;
        }
        return true;
    }

    private void butFlip_Click(object sender, EventArgs e)
    {
        if (!canRunScript()) return;
        AASendCommand(CMD.SCRIPT_RUN, "Image.FlipH");
        butImage.PerformClick();
    }

    private void butRunScript_Click(object sender, EventArgs e)
    {
        if (!canRunScript()) return;
        string scriptString = textScript.Text;
        AASendCommand(CMD.SCRIPT_RUN, scriptString);
    }

    private void butVar_Click(object sender, EventArgs e)
    {
        labVar.Text = "= " + AAReadScriptVariable(textBoxVar.Text);
    }

    private void butStatus_Click(object sender, EventArgs e)
    {
        int status = AASendCommand(CMD.SCRIPT_GETSTATUS, "");
        labStatus.Text = STS.toString(status);
    }

    private void pictureBox1_Paint(object sender, PaintEventArgs e)
    {
        e.Graphics.DrawImage(bmp, 0, 0);
    }

    private void butImage_Click(object sender, EventArgs e)
    {
        int w = AASendCommand(CMD.IMAGE_WIDTH, "");
        int h = AASendCommand(CMD.IMAGE_HEIGHT, "");
        labImage.Text = w.ToString() + " x " + h.ToString();
        if (w == 0) return;
        bmp = new Bitmap(w, h, PixelFormat.Format32bppRgb);
        for (int y = 0; y < h; y++)
        {
            for (int x = 0; x < w; x++)
            {
                int v = AAGetPixel(x, y);
                v = v / 50;            // this is a very simple visualization
                if (v > 255) v = 255;  // for 16 bit grayscale images
                bmp.SetPixel(x, h - 1 - y, Color.FromArgb(v * 0x010101));
                // bmp.SetPixel is VERY slow, use LockBits for real projects
                // https://stackoverflow.com/questions/18620277/writing-to-image-using-lock-bits-in-c-sharp
            }
        }
        pictureBox1.Invalidate();
    }

    private void butSetPixels_Click(object sender, EventArgs e)
    {
        int w = AASendCommand(CMD.IMAGE_WIDTH, "");
        int h = AASendCommand(CMD.IMAGE_HEIGHT, "");
        if (w == 0) return;
        for (int y = 0; y < h/2; y++)
        {
            for (int x = 0; x < w/2; x++)
            {
                double v = Math.Sqrt(x*x + y*y) * 30.0;
                AASetPixel(x, y, (int)v);
            }
        }
        AASendCommand(CMD.IMAGE_UPDATE, "");
    }

}

static class CMD
{
    public const int OK = 0xAA;
    public const int EXECUTE = 0;
    public const int TERMINATE = 1;
    public const int SCRIPT_RUN = 10;
    public const int SCRIPT_STOP = 11;
    public const int SCRIPT_GETSTATUS = 12;
    public const int SCRIPT_SHOW = 13;
    public const int SCRIPT_HIDE = 14;
    public const int SCRIPT_GETVAR = 20;
    public const int IMAGE_WIDTH = 30;
    public const int IMAGE_HEIGHT = 31;
    public const int IMAGE_UPDATE = 32;
    public const int IMAGE_PREPAREUNDO = 36; 
    public const int SELECT_INTERFACE_62 = 5;
    public const int SELECT_INTERFACE_60 = 4;
}

static class STS
{
    public const int IDLE = 0xA1;
    public const int RUNNING = 0xA2;
    public const int CAMWAIT = 0xA3;
    public const int TELEWAIT = 0xA4;
    public const int FOCUSERWAIT = 0xA5;
    public const int GUIDESTARWAIT = 0xA6;
    public static string toString(int status)
    {
        switch (status) {
            case IDLE: return "Idle";
            case RUNNING: return "Script running";
            case CAMWAIT: return "Camera exposing";
            case TELEWAIT: return "Telescope is moving";
            case FOCUSERWAIT: return "Focuser is moving";
            case GUIDESTARWAIT: return "Waiting for autoguide star";
            default: return ("Error!");
        }
    }
}

}
